@@ -35,7 +35,6 @@ Created 1/8/1996 Heikki Tuuri
 #include <my_sys.h>
 #include <deque>
 
-class MDL_context;
 class MDL_ticket;
 
 /** the first table or index ID for other than hard-coded system tables */
@@ -133,28 +132,13 @@ enum dict_table_op_t {
 @param[in]      table_op        operation to perform when opening
 @return table object after locking MDL shared
 @retval NULL if the table is not readable, or if trylock && MDL blocked */
-template<bool trylock>
+template<bool trylock, bool purge_thd= false>
 dict_table_t*
 dict_acquire_mdl_shared(dict_table_t *table,
                         THD *thd,
                         MDL_ticket **mdl,
                         dict_table_op_t table_op= DICT_TABLE_OP_NORMAL);
 
-/** Acquire MDL shared for the table name.
-@tparam trylock whether to use non-blocking operation
-@param[in,out]  table           table object
-@param[in,out]  mdl_context     MDL context
-@param[out]     mdl             MDL ticket
-@param[in]      table_op        operation to perform when opening
-@return table object after locking MDL shared
-@retval nullptr if the table is not readable, or if trylock && MDL blocked */
-template<bool trylock>
-__attribute__((nonnull, warn_unused_result))
-dict_table_t*
-dict_acquire_mdl_shared(dict_table_t *table,
-                        MDL_context *mdl_context, MDL_ticket **mdl,
-                        dict_table_op_t table_op);
-
 /** Look up a table by numeric identifier.
 @param[in]      table_id        table identifier
 @param[in]      dict_locked     data dictionary locked
@@ -162,6 +146,7 @@ dict_acquire_mdl_shared(dict_table_t *table,
 @param[in,out]  thd             background thread, or NULL to not acquire MDL
 @param[out]     mdl             mdl ticket, or NULL
 @return table, NULL if does not exist */
+template<bool purge_thd= false>
 dict_table_t*
 dict_table_open_on_id(table_id_t table_id, bool dict_locked,
                       dict_table_op_t table_op, THD *thd= nullptr,
@@ -436,6 +421,14 @@ dict_foreign_add_to_cache(
 	dict_err_ignore_t	ignore_err)
 				/*!< in: error to be ignored */
 	MY_ATTRIBUTE((nonnull(1), warn_unused_result));
+/*********************************************************************//**
+Checks if a table is referenced by foreign keys.
+@return TRUE if table is referenced by a foreign key */
+ibool
+dict_table_is_referenced_by_foreign_key(
+/*====================================*/
+	const dict_table_t*	table)	/*!< in: InnoDB table */
+	MY_ATTRIBUTE((nonnull, warn_unused_result));
 /**********************************************************************//**
 Replace the index passed in with another equivalent index in the
 foreign key lists of the table.
@@ -862,11 +855,8 @@ fil_space_t::flags  |     0     |    0    |     1      |    1
 ==================================================================
 @param[in]	table_flags	dict_table_t::flags
 @return tablespace flags (fil_space_t::flags) */
-UNIV_INLINE
-ulint
-dict_tf_to_fsp_flags(ulint table_flags)
-	MY_ATTRIBUTE((const));
-
+inline uint32_t dict_tf_to_fsp_flags(unsigned table_flags)
+  MY_ATTRIBUTE((const));
 
 /** Extract the ROW_FORMAT=COMPRESSED page size from table flags.
 @param[in]	flags	flags
@@ -1333,7 +1323,13 @@ class dict_sys_t
   std::atomic<ulonglong> latch_ex_wait_start;
 
   /** the rw-latch protecting the data dictionary cache */
-  alignas(CPU_LEVEL1_DCACHE_LINESIZE) IF_DBUG(srw_lock_debug,srw_lock) latch;
+  alignas(CPU_LEVEL1_DCACHE_LINESIZE) srw_lock latch;
+#ifdef UNIV_DEBUG
+  /** whether latch is being held in exclusive mode (by any thread) */
+  bool latch_ex;
+  /** number of S-latch holders */
+  Atomic_counter<uint32_t> latch_readers;
+#endif
 public:
   /** Indexes of SYS_TABLE[] */
   enum
@@ -1501,12 +1497,14 @@ public:
   }
 
 #ifdef UNIV_DEBUG
-  /** @return whether the current thread is holding the latch */
-  bool frozen() const { return latch.have_any(); }
-  /** @return whether the current thread is holding a shared latch */
-  bool frozen_not_locked() const { return latch.have_rd(); }
-  /** @return whether the current thread holds the exclusive latch */
-  bool locked() const { return latch.have_wr(); }
+  /** @return whether any thread (not necessarily the current thread)
+  is holding the latch; that is, this check may return false
+  positives */
+  bool frozen() const { return latch_readers || locked(); }
+  /** @return whether any thread (not necessarily the current thread)
+  is holding the exclusive latch; that is, this check may return false
+  positives */
+  bool locked() const { return latch_ex; }
 #endif
 private:
   /** Acquire the exclusive latch */
@@ -1521,7 +1519,13 @@ public:
   /** Exclusively lock the dictionary cache. */
   void lock(SRW_LOCK_ARGS(const char *file, unsigned line))
   {
-    if (!latch.wr_lock_try())
+    if (latch.wr_lock_try())
+    {
+      ut_ad(!latch_readers);
+      ut_ad(!latch_ex);
+      ut_d(latch_ex= true);
+    }
+    else
       lock_wait(SRW_LOCK_ARGS(file, line));
   }
 
@@ -1534,11 +1538,27 @@ public:
   ATTRIBUTE_NOINLINE void unfreeze();
 #else
   /** Unlock the data dictionary cache. */
-  void unlock() { latch.wr_unlock(); }
+  void unlock()
+  {
+    ut_ad(latch_ex);
+    ut_ad(!latch_readers);
+    ut_d(latch_ex= false);
+    latch.wr_unlock();
+  }
   /** Acquire a shared lock on the dictionary cache. */
-  void freeze() { latch.rd_lock(); }
+  void freeze()
+  {
+    latch.rd_lock();
+    ut_ad(!latch_ex);
+    ut_d(latch_readers++);
+  }
   /** Release a shared lock on the dictionary cache. */
-  void unfreeze() { latch.rd_unlock(); }
+  void unfreeze()
+  {
+    ut_ad(!latch_ex);
+    ut_ad(latch_readers--);
+    latch.rd_unlock();
+  }
 #endif
 
   /** Estimate the used memory occupied by the data dictionary
